@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2022 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,6 +10,7 @@
 
 from .common import PostProcessor
 from .. import util, formatter
+import json
 import sys
 import os
 
@@ -21,6 +22,9 @@ class MetadataPP(PostProcessor):
 
         mode = options.get("mode")
         cfmt = options.get("content-format") or options.get("format")
+        omode = "w"
+        filename = None
+
         if mode == "tags":
             self.write = self._write_tags
             ext = "txt"
@@ -41,10 +45,14 @@ class MetadataPP(PostProcessor):
                 cfmt = "\n".join(cfmt) + "\n"
             self._content_fmt = formatter.parse(cfmt).format_map
             ext = "txt"
+        elif mode == "jsonl":
+            self.write = self._write_json
+            self._json_encode = self._make_encoder(options).encode
+            omode = "a"
+            filename = "data.jsonl"
         else:
             self.write = self._write_json
-            self.indent = options.get("indent", 4)
-            self.ascii = options.get("ascii", False)
+            self._json_encode = self._make_encoder(options, 4).encode
             ext = "json"
 
         directory = options.get("directory")
@@ -53,7 +61,7 @@ class MetadataPP(PostProcessor):
             sep = os.sep + (os.altsep or "")
             self._metadir = util.expand_path(directory).rstrip(sep) + os.sep
 
-        filename = options.get("filename")
+        filename = options.get("filename", filename)
         extfmt = options.get("extension-format")
         if filename:
             if filename == "-":
@@ -74,29 +82,12 @@ class MetadataPP(PostProcessor):
             events = events.split(",")
         job.register_hooks({event: self.run for event in events}, options)
 
-        archive = options.get("archive")
-        if archive:
-            extr = job.extractor
-            archive = util.expand_path(archive)
-            archive_format = (
-                options.get("archive-prefix", extr.category) +
-                options.get("archive-format", "_MD_" + extr.archive_fmt))
-            try:
-                if "{" in archive:
-                    archive = formatter.parse(archive).format_map(
-                        job.pathfmt.kwdict)
-                self.archive = util.DownloadArchive(
-                    archive, archive_format, "_archive_metadata")
-            except Exception as exc:
-                self.log.warning(
-                    "Failed to open download archive at '%s' ('%s: %s')",
-                    archive, exc.__class__.__name__, exc)
-            else:
-                self.log.debug("Using download archive '%s'", archive)
-        else:
-            self.archive = None
-
+        self._init_archive(job, options, "_MD_")
         self.mtime = options.get("mtime")
+        self.omode = options.get("open", omode)
+        self.encoding = options.get("encoding", "utf-8")
+        self.private = options.get("private", False)
+        self.skip = options.get("skip", False)
 
     def run(self, pathfmt):
         archive = self.archive
@@ -106,12 +97,15 @@ class MetadataPP(PostProcessor):
         directory = self._directory(pathfmt)
         path = directory + self._filename(pathfmt)
 
+        if self.skip and os.path.exists(path):
+            return
+
         try:
-            with open(path, "w", encoding="utf-8") as fp:
+            with open(path, self.omode, encoding=self.encoding) as fp:
                 self.write(fp, pathfmt.kwdict)
         except FileNotFoundError:
             os.makedirs(directory, exist_ok=True)
-            with open(path, "w", encoding="utf-8") as fp:
+            with open(path, self.omode, encoding=self.encoding) as fp:
                 self.write(fp, pathfmt.kwdict)
 
         if archive:
@@ -194,11 +188,30 @@ class MetadataPP(PostProcessor):
             for taglist in taglists:
                 extend(taglist)
             tags.sort()
+        elif all(isinstance(e, dict) for e in tags):
+            taglists = tags
+            tags = []
+            extend = tags.extend
+            for tagdict in taglists:
+                extend([x for x in tagdict.values() if x is not None])
+            tags.sort()
 
         fp.write("\n".join(tags) + "\n")
 
     def _write_json(self, fp, kwdict):
-        util.dump_json(util.filter_dict(kwdict), fp, self.ascii, self.indent)
+        if not self.private:
+            kwdict = util.filter_dict(kwdict)
+        fp.write(self._json_encode(kwdict) + "\n")
+
+    @staticmethod
+    def _make_encoder(options, indent=None):
+        return json.JSONEncoder(
+            ensure_ascii=options.get("ascii", False),
+            sort_keys=options.get("sort", False),
+            separators=options.get("separators"),
+            indent=options.get("indent", indent),
+            check_circular=False, default=str,
+        )
 
 
 __postprocessor__ = MetadataPP
